# -*- coding: utf-8 -*-
from typing import List
from .versions.vfg_0_4_0 import Distribution


class ValidationError(Exception):
    pass


class InvalidVariableName(ValidationError, Exception):
    def __init__(self, which_var: str):
        super(Exception, self).__init__("Invalid variable name: '%s'" % which_var)


class InvalidVariableItemCount(ValidationError, Exception):
    def __init__(self, which_var: str):
        super(Exception, self).__init__(
            "Variable '%s' must have at least 1 value." % which_var
        )


class MissingVariable(ValidationError, Exception):
    def __init__(self):
        super(Exception, self).__init__("A factor must have at least one variable.")


class MissingProbability(ValidationError, Exception):
    def __init__(self):
        super(Exception, self).__init__(
            "A factor must have at least one probability value."
        )


class VariableMissingInVariableList(ValidationError, Exception):
    def __init__(self, which_var: str):
        super(Exception, self).__init__(
            "Factor variable '%s' is not defined in variables" % which_var
        )


class IncorrectProbabilityLength(ValidationError, Exception):
    def __init__(self, variables: List[str], expected_size: int, actual_size: int):
        super(Exception, self).__init__(
            "Factor variables '%s' have incorrect number of probability values. Expected %d, found %d"
            % (variables, expected_size, actual_size)
        )


class StrideMustSumToOneError(ValidationError, Exception):
    def __init__(
        self, distribution_type: Distribution, val_sum: float, elements: List[str]
    ):
        super(Exception, self).__init__(
            "Factor values for each category must sum to 1.0 for %s distributions. Found sum of %f for %s"
            % (distribution_type, val_sum, elements)
        )


class InvalidShapeError(ValidationError, Exception):
    def __init__(self, strides: List[int], values: List[float]):
        super(Exception, self).__init__(
            "Invalid shape. Found strides %s for Values %s; product of strides must equal length of elements"
            % (strides, values)
        )


class JsonSerializationError(ValidationError, Exception):
    pass
